use warnings;
use strict;
use Carp;

BEGIN { require 't/.lib.t' }
plan tests => 35;

gpg_config({ homedir => undef });
throws_ok { gpg_list() } qr/homedir.*gpg_config/;
gpg_config({ homedir => q{} });
throws_ok { gpg_list() } qr/homedir.*gpg_config/;
gpg_config({ homedir => $HOMEDIR });

system("chmod 0700 \Q$HOMEDIR\E &>/dev/null; rm -rf \Q$HOMEDIR\E");
init_gpg_homedir();
lives_ok  { gpg_list() } 'selftest: init_gpg_homedir() works good without dir';
init_gpg_homedir();
lives_ok  { gpg_list() } 'selftest: init_gpg_homedir() works good with dir';

throws_ok { gpg_import($SEC_KEY{$KEY_ID[0]})    } qr/secret.*prohibited/;
lives_ok  { gpg_import($SEC_KEY{$KEY_ID[0]}, 1) } 'forcing secret key import';
my $encr =    gpg_encrypt('test', [$KEY_ID[0]]);
my $sign = gpg_detachsign('test', [$KEY_ID[0]]);

# No homedir.
system("chmod 0700 \Q$HOMEDIR\E &>/dev/null; rm -rf \Q$HOMEDIR\E");
### tests disabled because newer gpg automatically create homedir
#throws_ok { gpg_list()                          } qr/file open error/;
#throws_ok { gpg_import($PUB_KEY{$KEY_ID[0]})    } qr/file open error/;
#throws_ok { gpg_export($KEY_ID[0])              } qr/file open error/;
#throws_ok { gpg_delete($KEY_ID[0])              } qr/file open error/;
#throws_ok { gpg_encrypt(q{}, [$KEY_ID[0]])      } qr/file open error/;
#throws_ok { gpg_encrypt_sign(q{}, [$KEY_ID[0]]) } qr/file open error/;
#throws_ok { gpg_detachsign(q{})                 } qr/file open error/;
#throws_ok { gpg_decrypt($encr)                  } qr/file open error/;
lives_ok  { gpg_verify('test', $sign)           } 'no way to detect err in verify';

# Homedir with unsafe permissions.
mkdir $HOMEDIR; chmod 0755, $HOMEDIR;
throws_ok { gpg_list()                          } qr/unsafe permissions/;
throws_ok { gpg_import($PUB_KEY{$KEY_ID[0]})    } qr/unsafe permissions/;
throws_ok { gpg_export($KEY_ID[0])              } qr/unsafe permissions/;
throws_ok { gpg_delete($KEY_ID[0])              } qr/unsafe permissions/;
throws_ok { gpg_encrypt(q{}, [$KEY_ID[0]])      } qr/unsafe permissions/;
throws_ok { gpg_encrypt_sign(q{}, [$KEY_ID[0]]) } qr/unsafe permissions/;
throws_ok { gpg_detachsign(q{})                 } qr/unsafe permissions/;
throws_ok { gpg_decrypt($encr)                  } qr/unsafe permissions/;
throws_ok { gpg_verify('test', $sign)           } qr/unsafe permissions/;

chmod 0700, $HOMEDIR;
lives_ok  { gpg_list() } 'empty homedir with safe permissions';

# Files in homedir with permissions=0.
chmod 0, $_ for glob("\Q$HOMEDIR\E/*");
throws_ok { gpg_list()                          } qr/Permission denied/;
throws_ok { gpg_import($PUB_KEY{$KEY_ID[0]})    } qr/can't open/;
throws_ok { gpg_export($KEY_ID[0])              } qr/can't open/;
throws_ok { gpg_delete($KEY_ID[0])              } qr/can't open/;
throws_ok { gpg_encrypt(q{}, [$KEY_ID[0]])      } qr/can't open/;
throws_ok { gpg_encrypt_sign(q{}, [$KEY_ID[0]]) } qr/can't open/;
throws_ok { gpg_detachsign(q{})                 } qr/can't open/;
throws_ok { gpg_decrypt($encr)                  } qr/can't open/;
lives_ok  { gpg_verify('test', $sign)           } 'no way to detect err in verify';

# Homedir with permissions=0.
chmod 0, $HOMEDIR;
throws_ok { gpg_list()                          } qr/general error/;
throws_ok { gpg_import($PUB_KEY{$KEY_ID[0]})    } qr/general error/;
throws_ok { gpg_export($KEY_ID[0])              } qr/general error/;
throws_ok { gpg_delete($KEY_ID[0])              } qr/general error/;
throws_ok { gpg_encrypt(q{}, [$KEY_ID[0]])      } qr/general error/;
throws_ok { gpg_encrypt_sign(q{}, [$KEY_ID[0]]) } qr/general error/;
throws_ok { gpg_detachsign(q{})                 } qr/general error/;
throws_ok { gpg_decrypt($encr)                  } qr/general error/;
lives_ok  { gpg_verify('test', $sign)           } 'no way to detect err in verify';

